@@ -1,13 +1,13 @@
-use super::lsystems_3d::Turtle;
-use super::lsystems_3d::TurtleCommand;
-use super::lsystems_3d::TurtleRules;
-use super::TBError;
 use crate::toxicblend_pb::Command as PB_Command;
 use crate::toxicblend_pb::Face as PB_Face;
 use crate::toxicblend_pb::KeyValuePair as PB_KeyValuePair;
 use crate::toxicblend_pb::Model as PB_Model;
 use crate::toxicblend_pb::Reply as PB_Reply;
 use crate::toxicblend_pb::Vertex as PB_Vertex;
+use crate::{
+    lsystems_3d::{Turtle, TurtleCommand, TurtleRules},
+    TBError,
+};
 
 use cgmath::{EuclideanSpace, Rad};
 use std::collections::HashMap;
@@ -536,7 +536,7 @@ fn build_output_bp_model(
 }
 
 pub fn command(
-    a_command: &PB_Command,
+    a_command: PB_Command,
     options: HashMap<String, String>,
 ) -> Result<PB_Reply, TBError> {
     println!(
@@ -610,7 +610,7 @@ pub fn command(
         ))),
     }?;
 
-    let pb_model = build_output_bp_model(a_command, mesh)?;
+    let pb_model = build_output_bp_model(&a_command, mesh)?;
     println!("pb_model.vertices.len() {}", pb_model.vertices.len());
     println!("pb_model.faces.len() {}", pb_model.faces.len());
 
