@@ -1,10 +1,10 @@
-use super::TBError;
 use crate::toxicblend_pb::Command as PB_Command;
 use crate::toxicblend_pb::Face as PB_Face;
 use crate::toxicblend_pb::KeyValuePair as PB_KeyValuePair;
 use crate::toxicblend_pb::Model as PB_Model;
 use crate::toxicblend_pb::Reply as PB_Reply;
 use crate::toxicblend_pb::Vertex as PB_Vertex;
+use crate::TBError;
 use itertools::Itertools;
 use linestring::linestring_3d;
 use rayon::prelude::*;
@@ -412,7 +412,7 @@ fn build_bp_model(
 }
 
 pub fn command(
-    a_command: &PB_Command,
+    a_command: PB_Command,
     options: HashMap<String, String>,
 ) -> Result<PB_Reply, TBError> {
     println!(
@@ -455,7 +455,7 @@ pub fn command(
 
     let lines = find_linestrings(&a_command.models[0])?;
     let lines = simplify_rdp_percent(&a_command.models[0], distance, lines)?;
-    let model = build_bp_model(a_command, lines)?;
+    let model = build_bp_model(&a_command, lines)?;
 
     let mut reply = PB_Reply {
         options: vec![PB_KeyValuePair {
