@@ -18,17 +18,18 @@
 #[cfg(feature = "saft")]
 extern crate saft_cr as saft;
 mod cmd_2d_outline;
-// Replacing building_blocks with fast_surface_nets, but keeping the files for now
+// Replacing building_blocks with fast_surface_nets, but keeping the bb files for now
 //mod cmd_bb_sdf;
 //mod cmd_bb_voxel;
 mod cmd_centerline;
+mod cmd_fsn_mavoxel;
 mod cmd_fsn_sdf;
 mod cmd_fsn_voxel;
 mod cmd_knife_intersect;
 mod cmd_lsystems;
 #[cfg(feature = "saft")]
 mod cmd_saft_voxel;
-//#[cfg(feature = "saft")]
+//#[cfg(feature = "saft")] // not implemented yet
 //mod cmd_saft_sdf;
 mod cmd_simplify;
 mod cmd_voronoi;
@@ -208,7 +209,7 @@ impl ToxicBlendService for TheToxicBlendService {
         &self,
         request: PB_Request<PB_Command>,
     ) -> Result<PB_Response<PB_Reply>, PB_Status> {
-        let a_command = request.get_ref();
+        let a_command = request.into_inner();
         let options_map = options_to_map(&a_command.options);
 
         let rv = match a_command.command.as_str() {
@@ -223,6 +224,7 @@ impl ToxicBlendService for TheToxicBlendService {
             ))),
             //"voxel_bb" => cmd_bb_voxel::command(a_command, options_map),
             "voxel_fsn" => cmd_fsn_voxel::command(a_command, options_map),
+            "mavoxel_fsn" => cmd_fsn_mavoxel::command(a_command, options_map),
             #[cfg(feature = "saft")]
             "voxel_saft" => cmd_saft_voxel::command(a_command, options_map),
             #[cfg(not(feature = "saft"))]
@@ -238,7 +240,7 @@ impl ToxicBlendService for TheToxicBlendService {
             "sdf_saft" => Err(TBError::NotImplemented(String::from(
                 "The sdf backend 'saft' is not implemented (yet)",
             ))),
-            _ => Err(TBError::UnknownCommand(a_command.command.clone())),
+            _ => Err(TBError::UnknownCommand(a_command.command)),
         };
         // convert TBError to a valid Error message
         if let Err(err) = rv {
